/* 
 * This file is part of "Modulos de entrenamiento para FPGAs"
 * Copyright (c) 2018 Miguel Angel Rodriguez Jodar.
 * 
 * This program is free software: you can redistribute it and/or modify  
 * it under the terms of the GNU General Public License as published by  
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

/*
   EJEMPLO 3
   ---------
   
   Este código contiene un ejemplo de uso de los módulos de entrenamiento para FPGA, y
   está escrito para ser incluido dentro de un módulo (ver tld_modulos_entrenamiento_*.v)
   No es por tanto un código Verilog completo.
   
   Dentro de este fichero se pueden definir wire y reg, así como instanciar al módulo
   "display" que es el módulo principal del conjunto de módulos de entrenamiento.
   
   Dentro de este fichero no se pueden definir otros módulos o entidades, ya que este 
   fichero en sí está dentro de un módulo preexistente.

   En este ejemplo usamos un contador que cambia el estado de un flipflop cada
   12,5 millones de ciclos de reloj (recuerda que usamos un reloj de 25 MHz para estos
   diseños, generado en la propia FPGA, así que esta cantidad de ciclos se corresponde a
   medio segundo). La salida del flipflop se conecta al led de más a la izquierda, con lo que
   vemos en tiempo real como cambia ese flipflop, es decir, vemos cómo parpadea el led.
*/   
module ejemplo03 (
input wire clk,
output wire [7:0]leds = 8'b00000001
);


  reg led_parpadeante = 1'b0;   // el estado del led que parpadea se guarda en este flipflop
  reg [23:0] contador = 24'd0;  // contador que cuenta de 0 hasta 12500000 (medio segundo, a 25 MHz)

  assign leds[0]= led_parpadeante;
  
  reloj25mhz r25mhz (.inclk0(clk),.c0(clk25m)); //sacamos 25Mhz de la señal de reloj principal.
  
  always @(posedge clk25m) begin   // en cada ciclo de reloj del reloj de 25 MHz hacemos lo siguiente:
    if (contador == 24'd12500000) begin  // si hemos llegado a medio segundo de tiempo...
      contador <= 24'd0;                 //    ponemos el contador a 0, e
      led_parpadeante <= ~led_parpadeante; //  invertimos el estado del flipflop
    end
    else                                 // en otro caso
      contador <= contador + 24'd1;      //    incrementamos el contador
  end
  endmodule