/* 
 * This file is part of "Modulos de entrenamiento para FPGAs"
 * Copyright (c) 2018 Miguel Angel Rodriguez Jodar.
 * 
 * This program is free software: you can redistribute it and/or modify  
 * it under the terms of the GNU General Public License as published by  
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

/*
   EJEMPLO 4
   ---------
   
   Este código contiene un ejemplo de uso de los módulos de entrenamiento para FPGA, y
   está escrito para ser incluido dentro de un módulo (ver tld_modulos_entrenamiento_*.v)
   No es por tanto un código Verilog completo.
   
   Dentro de este fichero se pueden definir wire y reg, así como instanciar al módulo
   "display" que es el módulo principal del conjunto de módulos de entrenamiento.
   
   Dentro de este fichero no se pueden definir otros módulos o entidades, ya que este 
   fichero en sí está dentro de un módulo preexistente.

   En este ejemplo vemos una variación del EJEMPLO 3. Aquí, en lugar de invertir el valor de un
   bit en un flipflop, lo que hacemos es rotar el contenido de un registro de 8 bits a la derecha.
   El registro inicialmente tiene el valor 10000000. Al rotarse, este registro irá tomando los valores
   01000000, 00100000, 00010000, 00001000, 00000100, 00000010, 000000001, y de nuevo 10000000.
   Al mostrarse en los leds, el efecto es el de una luz moviéndose de izquierda a derecha y vuelta
   a empezar
*/   
module ejemplo04 
(
input wire clk,
output wire [7:0]leds //= 8'b10000000)
);
  reloj25mhz r25mhz (.inclk0(clk),.c0(clk25m));

  reg [7:0] leds_reg = 8'b10000000;   
  reg [23:0] contador = 24'd0;  // contador que cuenta de 0 hasta 12500000 (medio segundo, a 25 MHz)
  assign leds = leds_reg;
  
  always @(posedge clk25m) begin   // en cada ciclo de reloj del reloj de 25 MHz hacemos lo siguiente:
    if (contador == 24'd12500000) begin  // si hemos llegado a medio segundo de tiempo...
      contador <= 24'd0;                 //    ponemos el contador a 0, e
      leds_reg <= {leds_reg[0], leds_reg[7:1]};      // Rotamos el valor de leds un bit a la derecha (bit 7 al 6, 6 al 5, etc. El bit 0 pasa al bit 7.
    end
    else                                 // en otro caso
      contador <= contador + 24'd1;      //    incrementamos el contador
  end
  endmodule