/* 
 * This file is part of "Modulos de entrenamiento para FPGAs"
 * Copyright (c) 2018 Miguel Angel Rodriguez Jodar.
 * 
 * This program is free software: you can redistribute it and/or modify  
 * it under the terms of the GNU General Public License as published by  
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

/*
   EJEMPLO 5
   ---------
   
   Este código contiene un ejemplo de uso de los módulos de entrenamiento para FPGA, y
   está escrito para ser incluido dentro de un módulo (ver tld_modulos_entrenamiento_*.v)
   No es por tanto un código Verilog completo.
   
   Dentro de este fichero se pueden definir wire y reg, así como instanciar al módulo
   "display" que es el módulo principal del conjunto de módulos de entrenamiento.
   
   Dentro de este fichero no se pueden definir otros módulos o entidades, ya que este 
   fichero en sí está dentro de un módulo preexistente.

   En este ejemplo vemos una variación del EJEMPLO 4. Usamos el interruptor de más a la izquierda,
   que corresponde al bit 7 de la señal "switches" para elegir si rotamos el registro "leds" a derecha
   (interruptor en OFF) o a la izquierda (interruptor en ON).
   Para hacer un poco más llamativo este efecto, se ha cambiado el valor límite de la cuenta al equivalente
   en ciclos de reloj de un cuarto de segundo ( 25000000 * 1/4 = 6250000 )
*/   
module ejemplo05 
(
input wire clk,
output wire [7:0]leds //= 8'b10000000
);


  wire [7:0] switches;
  reg [7:0] leds_reg = 8'b10000000;   
  assign leds = leds_reg;
  reg [23:0] contador = 24'd0;  // contador que cuenta de 0 hasta 6250000 (1/4 segundo, a 25 MHz)
 
 reloj25mhz r25mhz (.inclk0(clk),.c0(clk25m));
 
  always @(posedge clk25m) begin   // en cada ciclo de reloj del reloj de 25 MHz hacemos lo siguiente:
    if (contador == 24'd6250000) begin   // si hemos llegado a un cuarto de segundo de tiempo...
      contador <= 24'd0;                 //    ponemos el contador a 0, y
      if (switches[7] == 1'b0)           //    miramos el estado del switch 7, para rotar a derecha (0) o izquierda (1)
        leds_reg <= {leds_reg[0], leds_reg[7:1]};    //      rotamos a la derecha si es 0
      else                               //      si no,
        leds_reg <= {leds_reg[6:0], leds_reg[7]};    //      entonces rotamos a la izquierda
    end
    else                                 // en otro caso
      contador <= contador + 24'd1;      //    incrementamos el contador
  end
  
  endmodule