/* 
 * This file is part of "Modulos de entrenamiento para FPGAs"
 * Copyright (c) 2018 Miguel Angel Rodriguez Jodar.
 * 
 * This program is free software: you can redistribute it and/or modify  
 * it under the terms of the GNU General Public License as published by  
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
*/


/*  Añadimos un cronómetro al EJEMPLO 6. Usando algunos de los dígitos del display hexadecimal,
   implementaremos un cronómetro simple que sea capaz de contar hasta 65536 segundos (mostrados
   en hexadecimal en el grupo de digitos centrales) y un dígito para las décimas de segundo.
*/   
module ejemplo07 
(
input wire clk,
output wire [7:0]leds, //= 8'b10000000

	output wire [6:0] seg_0,
	output wire [6:0] seg_1,
	output wire [6:0] seg_2,
	output wire [6:0] seg_3,
	output wire [6:0] seg_4,
	output wire [6:0] seg_5,
	output wire	[6:0] seg_6,
	output wire [6:0] seg_7,
	output reg [7:0] punto
);
  reg [7:0]leds_reg = 8'b10000000;
  assign leds = leds_reg;
  reg sentido = 1'b0;  // en este flipflop guardamos el sentido: 0=derecha, 1=izquierda   
  reg [23:0] contador = 24'd0;  // contador que cuenta de 0 hasta 2500000 (una décima de segundo, a 25 MHz)

  // Valores iniciales del cronómetro
  reg [3:0] decimas = 4'd0;
  reg [15:0] segundos = 16'h0000;

  reloj25mhz r25mhz (.inclk0(clk),.c0(clk25m));

  always @(posedge clk25m) begin   // en cada ciclo de reloj del reloj de 25 MHz hacemos lo siguiente:
    if (contador == 24'd2500000) begin   // si hemos llegado a una décima de segundo de tiempo...
      contador <= 24'd0;                 //    ponemos el contador a 0, y

      // Gestión del cronómetro
      if (decimas == 4'd9) begin         // Vamos a pasar de 9 a 10 en las decimas? Si es que si,
        decimas <= 4'd0;                 //    ponemos las décimas a 0
        segundos <= segundos + 16'd1;    //    E incrementamos el contador de segundos.
      end                                
      else                               // Si es que no, pues entonces
        decimas <= decimas + 4'd1;       //    incrementamos las décimas de segundo
        
      // Gestión de la tira de leds
      if (sentido == 1'b0)               //    miramos en qué sentido hemos de rotar
        leds_reg <= {leds_reg[0], leds_reg[7:1]};    //      rotamos a la derecha si es 0
      else                               //      si no,
        leds_reg <= {leds_reg[6:0], leds_reg[7]};    //      entonces rotamos a la izquierda
      if (leds_reg[1] == 1'b1 && sentido == 1'b0)       // si estamos a punto de llegar a la esquina derecha y estábamos rotando a la derecha...
        sentido <= 1'b1;                            //    pasamos a rotar a la izquierda
      else if (leds_reg[6] == 1'b1 && sentido == 1'b1)  // si no, si estamos a punto de llegar a la esquina izquierda y estábamos rotando a la izquierda...
        sentido <= 1'b0;                            //    pasamos a rotar a la derecha
    end
    else                                 // en otro caso
      contador <= contador + 24'd1;      //    incrementamos el contador
    
	 
	 punto = ~8'b00000000; //apagamos todos los puntos de los BCD 

   end
  
llamardisplay display_00 (.binario(segundos[15:12]), .display(seg_0));
llamardisplay display_01 (.binario(segundos[11:8]), .display(seg_1));
llamardisplay display_02 (.binario(segundos[7:4]), .display(seg_2));
llamardisplay display_03 (.binario(segundos[3:0]), .display(seg_3));
llamardisplay display_04 (.binario(decimas), .display(seg_4));
llamardisplay display_05 (.binario(0), .display(seg_5));
llamardisplay display_06 (.binario(0), .display(seg_6));
llamardisplay display_07 (.binario(0), .display(seg_7));

 endmodule